// @(#)root/proof:$Id: TProof.h 31520 2009-12-03 12:58:41Z brun $
// Author: Fons Rademakers   21/01/2010

/*************************************************************************
 * Copyright (C) 1995-2010, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifndef ROOT_TPackageManager
#define ROOT_TPackageManager


//////////////////////////////////////////////////////////////////////////
//                                                                      //
// TPackageManager                                                      //
//                                                                      //
// This class manages PAR (PROOF ARchive) files. PAR files originally   //
// were developed as a way to distibute external user code to PROOF     //
// workers, but their use is general and PAR files can also be          //
// used in local and Grid sessions.                                     //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#ifndef ROOT_TObject
#include "TObject.h"
#endif
#ifndef ROOT_TString
#include "TString.h"
#endif

class TList;
class THashList;
class TProof;
class TMD5;
class TProofLockPath;


class TPackage : public TObject {

private:
   TString   fName;   // package name
   TMD5     *fMd5;    // package md5 checksum

   TPackage() : fName(), fMd5(0) { }
   TPackage(const char *name);
   TPackage(const TPackage &);             // not implemented
   TPackage& operator=(const TPackage &);  // not implemented
   virtual ~TPackage();

public:
   const char *GetName() const { return fName; }
   const TMD5 *GetMD5() const { return fMd5; }

   //ClassDef(TPackage,1)   // PAR package
};



class TPackageManager : public TObject {

private:
   // also in TProof.h (see where to move it)
   enum EBuildPackageOpt {
      kDontBuildOnClient   = -2,
      kBuildOnSlavesNoWait = -1,
      kBuildAll            = 0,
      kCollectBuildResults = 1
   };
   enum EUploadPackageOpt {
      kUntar               = 0x0,  //Untar over existing dir [default]
      kRemoveOld           = 0x1   //Remove existing dir with same name
   };

   TList          *fPackages;             // list of packages
   TProof         *fProof;                //! active PROOF instance
   TString         fPackageDir;           //!package directory (used on client)
   THashList      *fGlobalPackageDirList; //!list of directories containing global packages libs
   TProofLockPath *fPackageLock;          //!package lock
   TList          *fEnabledPackagesOnClient; //!list of packages enabled on client
   TList          *fAvailablePackages;    //!list of available packages
   TList          *fEnabledPackages;      //!list of enabled packages

   TPackageManager(const TPackageManager &);            // not implemented
   TPackageManager& operator=(const TPackageManager &); // not implemented

   Int_t    BuildPackage(const char *package, EBuildPackageOpt opt = kBuildAll);
   Int_t    BuildPackageOnClient(const TString &package);
   Int_t    LoadPackage(const char *package, Bool_t notOnClient = kFALSE);
   Int_t    LoadPackageOnClient(const TString &package);
   Int_t    UnloadPackage(const char *package);
   Int_t    UnloadPackageOnClient(const char *package);
   Int_t    UnloadPackages();
   Int_t    UploadPackageOnClient(const TString &package, EUploadPackageOpt opt, TMD5 *md5);
   Int_t    DisablePackage(const char *package);
   Int_t    DisablePackageOnClient(const char *package);
   Int_t    DisablePackages();

public:
   TPackageManager() : fPackages(0), fProof(0) { }
   virtual ~TPackageManager();

   void   AddPackage(const char *name);
   Bool_t RemovePackage(const char *name);
   const TPackage *FindPackage(const char *name);

   TList *GetListOfPackages();
   TList *GetListOfEnabledPackages();
   void   ShowPackages(Bool_t all = kFALSE);
   void   ShowEnabledPackages(Bool_t all = kFALSE);
   Int_t  ClearPackages();
   Int_t  ClearPackage(const char *package);
   Int_t  EnablePackage(const char *package, Bool_t notOnClient = kFALSE);
   Int_t  UploadPackage(const char *par, EUploadPackageOpt opt = kUntar);

   void   SetProof(TProof *proof, Bool_t sync = kTRUE);
   void   SyncProof();

   void   Print(Option_t *opt = "") const;

   //ClassDef(TPackageManager,1)  // Manager of PAR packages
};

R__EXTERN TPackageManager *gPackageMgr;

#endif
